@@ -1,4 +1,4 @@
-/* $Revision: #11 $$Date: 2002/09/18 $$Author: wsnyder $
+/* $Revision: #12 $$Date: 2002/11/06 $$Author: wsnyder $
  ************************************************************************
  *
  * THIS MODULE IS PUBLICLY LICENSED
@@ -29,36 +29,79 @@
 //**********************************************************************
 // Opening
 
-void sp_log_file::open (const char *filename)
-    /* Open main logfile */
-{
+void sp_log_file::open (const char *filename, int append) {
+    // Open main logfile
+    m_filename = filename;
+    m_splitNum = 0;
+    open_int(filename, append);
+}
+
+void sp_log_file::open_int (string filename, int append) {
+    // Internal open, used also for split
     this->close();
-    std::ofstream::open (filename);
-    if (0/*FIX*/) {
-	//UERROR("Can't write "<<filename<<endl);
-	printf ("Can't write %s\n", filename);
-	return;
-    }
+    std::ofstream::open (filename.c_str(), append);
     m_isOpen = true;
 }
 
 //**********************************************************************
 // Closing
 
-void sp_log_file::close (void)
-{
+void sp_log_file::close() {
+    end_redirect();
+    close_int();
+}
+
+void sp_log_file::close_int() {
     if (m_isOpen) {
-	end_redirect();
 	std::ofstream::close();
 	m_isOpen = false;
     }
 }
 
+//**********************************************************************
+// Split
+
+void sp_log_file::split_now () {
+    close_int();
+
+    // We rename the first file, so it will be obvious we rolled.
+    // This also has the nice effect of insuring downstream tools notice all revs.
+    if (m_splitNum==0) {
+	string newname = split_name(0);
+	rename (m_filename.c_str(), newname.c_str());
+	// We'll just ignore if there's an error with the rename
+    }
+    m_splitNum++;
+
+    open_int(split_name(m_splitNum));
+}
+
+string sp_log_file::split_name (unsigned suffixNum) {
+    string filename = m_filename;
+    char rollnum[10];
+    sprintf(rollnum, "_%03d", suffixNum);
+    unsigned pos = filename.rfind(".log");
+    if (pos == filename.length()-4) {
+	// Foo.log -> Foo_###.log
+	filename.erase(pos);
+	filename = filename + rollnum + ".log";
+    } else {
+	// Foo -> Foo_###
+	filename += rollnum;
+    }
+    return filename;
+}
+
+void sp_log_file::split_check () {
+    if (isOpen() && m_splitSize && (tellp() > m_splitSize)) {
+	split_now();
+    }
+}
+
 //**********************************************************************
 // Redirection
 
-void sp_log_file::redirect_cout (void)
-{
+void sp_log_file::redirect_cout() {
     if (m_strmOldCout) {
 	end_redirect();
     }
@@ -72,8 +115,7 @@ void sp_log_file::redirect_cout (void)
     std::cerr.rdbuf (m_tee);
 }
 
-void sp_log_file::end_redirect (void)
-{
+void sp_log_file::end_redirect() {
     if (m_strmOldCout) {
 	std::cout.rdbuf (m_strmOldCout);
 	std::cerr.rdbuf (m_strmOldCerr);
@@ -86,8 +128,7 @@ void sp_log_file::end_redirect (void)
 //**********************************************************************
 // C compatibility
 
-extern "C" void sp_log_printf(const char *format, ...)
-{
+extern "C" void sp_log_printf(const char *format, ...) {
     va_list ap;
     va_start (ap, format);
     std::cout.vform (format, ap);
@@ -97,8 +138,7 @@ extern "C" void sp_log_printf(const char *format, ...)
 
 #ifdef SP_LOG_MAIN
 //make -k stream && ./stream && echo "----" && cat sim.log
-int main ()
-{
+int main () {
     sp_log_file simlog ("sim.log");
 //    sp_log_file simlog;
 //    simlog.open ("sim.log");
